import typing

from kubernetes import client
from kuber import kube_api as _kube_api

from kuber import definitions as _kuber_definitions
from kuber.v1_15.meta_v1 import LabelSelector
from kuber.v1_15.meta_v1 import ListMeta
from kuber.v1_15.meta_v1 import ObjectMeta


class AggregationRule(_kuber_definitions.Definition):
    """
    AggregationRule describes how to locate ClusterRoles to
    aggregate into the ClusterRole
    """

    def __init__(
        self,
        cluster_role_selectors: typing.List["LabelSelector"] = None,
    ):
        """Create AggregationRule instance."""
        super(AggregationRule, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="AggregationRule"
        )
        self._properties = {
            "clusterRoleSelectors": cluster_role_selectors
            if cluster_role_selectors is not None
            else [],
        }
        self._types = {
            "clusterRoleSelectors": (list, LabelSelector),
        }

    @property
    def cluster_role_selectors(self) -> typing.List["LabelSelector"]:
        """
        ClusterRoleSelectors holds a list of selectors which will be
        used to find ClusterRoles and create the rules. If any of
        the selectors match, then the ClusterRole's permissions will
        be added
        """
        return typing.cast(
            typing.List["LabelSelector"],
            self._properties.get("clusterRoleSelectors"),
        )

    @cluster_role_selectors.setter
    def cluster_role_selectors(
        self, value: typing.Union[typing.List["LabelSelector"], typing.List[dict]]
    ):
        """
        ClusterRoleSelectors holds a list of selectors which will be
        used to find ClusterRoles and create the rules. If any of
        the selectors match, then the ClusterRole's permissions will
        be added
        """
        cleaned: typing.List[LabelSelector] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    LabelSelector,
                    LabelSelector().from_dict(item),
                )
            cleaned.append(typing.cast(LabelSelector, item))
        self._properties["clusterRoleSelectors"] = cleaned

    def __enter__(self) -> "AggregationRule":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ClusterRole(_kuber_definitions.Resource):
    """
    ClusterRole is a cluster level, logical grouping of
    PolicyRules that can be referenced as a unit by a
    RoleBinding or ClusterRoleBinding.
    """

    def __init__(
        self,
        aggregation_rule: "AggregationRule" = None,
        metadata: "ObjectMeta" = None,
        rules: typing.List["PolicyRule"] = None,
    ):
        """Create ClusterRole instance."""
        super(ClusterRole, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="ClusterRole"
        )
        self._properties = {
            "aggregationRule": aggregation_rule
            if aggregation_rule is not None
            else AggregationRule(),
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "rules": rules if rules is not None else [],
        }
        self._types = {
            "aggregationRule": (AggregationRule, None),
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "rules": (list, PolicyRule),
        }

    @property
    def aggregation_rule(self) -> "AggregationRule":
        """
        AggregationRule is an optional field that describes how to
        build the Rules for this ClusterRole. If AggregationRule is
        set, then the Rules are controller managed and direct
        changes to Rules will be stomped by the controller.
        """
        return typing.cast(
            "AggregationRule",
            self._properties.get("aggregationRule"),
        )

    @aggregation_rule.setter
    def aggregation_rule(self, value: typing.Union["AggregationRule", dict]):
        """
        AggregationRule is an optional field that describes how to
        build the Rules for this ClusterRole. If AggregationRule is
        set, then the Rules are controller managed and direct
        changes to Rules will be stomped by the controller.
        """
        if isinstance(value, dict):
            value = typing.cast(
                AggregationRule,
                AggregationRule().from_dict(value),
            )
        self._properties["aggregationRule"] = value

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def rules(self) -> typing.List["PolicyRule"]:
        """
        Rules holds all the PolicyRules for this ClusterRole
        """
        return typing.cast(
            typing.List["PolicyRule"],
            self._properties.get("rules"),
        )

    @rules.setter
    def rules(self, value: typing.Union[typing.List["PolicyRule"], typing.List[dict]]):
        """
        Rules holds all the PolicyRules for this ClusterRole
        """
        cleaned: typing.List[PolicyRule] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    PolicyRule,
                    PolicyRule().from_dict(item),
                )
            cleaned.append(typing.cast(PolicyRule, item))
        self._properties["rules"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the ClusterRole in the currently
        configured Kubernetes cluster.
        """
        names = ["create_namespaced_cluster_role", "create_cluster_role"]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the ClusterRole in the currently
        configured Kubernetes cluster.
        """
        names = ["replace_namespaced_cluster_role", "replace_cluster_role"]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the ClusterRole in the currently
        configured Kubernetes cluster.
        """
        names = ["patch_namespaced_cluster_role", "patch_cluster_role"]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the ClusterRole from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_cluster_role",
            "read_cluster_role",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the ClusterRole from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_cluster_role",
            "delete_cluster_role",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "ClusterRole":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ClusterRoleBinding(_kuber_definitions.Resource):
    """
    ClusterRoleBinding references a ClusterRole, but not contain
    it.  It can reference a ClusterRole in the global namespace,
    and adds who information via Subject.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        role_ref: "RoleRef" = None,
        subjects: typing.List["Subject"] = None,
    ):
        """Create ClusterRoleBinding instance."""
        super(ClusterRoleBinding, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="ClusterRoleBinding"
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "roleRef": role_ref if role_ref is not None else RoleRef(),
            "subjects": subjects if subjects is not None else [],
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "roleRef": (RoleRef, None),
            "subjects": (list, Subject),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def role_ref(self) -> "RoleRef":
        """
        RoleRef can only reference a ClusterRole in the global
        namespace. If the RoleRef cannot be resolved, the Authorizer
        must return an error.
        """
        return typing.cast(
            "RoleRef",
            self._properties.get("roleRef"),
        )

    @role_ref.setter
    def role_ref(self, value: typing.Union["RoleRef", dict]):
        """
        RoleRef can only reference a ClusterRole in the global
        namespace. If the RoleRef cannot be resolved, the Authorizer
        must return an error.
        """
        if isinstance(value, dict):
            value = typing.cast(
                RoleRef,
                RoleRef().from_dict(value),
            )
        self._properties["roleRef"] = value

    @property
    def subjects(self) -> typing.List["Subject"]:
        """
        Subjects holds references to the objects the role applies
        to.
        """
        return typing.cast(
            typing.List["Subject"],
            self._properties.get("subjects"),
        )

    @subjects.setter
    def subjects(self, value: typing.Union[typing.List["Subject"], typing.List[dict]]):
        """
        Subjects holds references to the objects the role applies
        to.
        """
        cleaned: typing.List[Subject] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    Subject,
                    Subject().from_dict(item),
                )
            cleaned.append(typing.cast(Subject, item))
        self._properties["subjects"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the ClusterRoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = [
            "create_namespaced_cluster_role_binding",
            "create_cluster_role_binding",
        ]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the ClusterRoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = [
            "replace_namespaced_cluster_role_binding",
            "replace_cluster_role_binding",
        ]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the ClusterRoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = ["patch_namespaced_cluster_role_binding", "patch_cluster_role_binding"]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the ClusterRoleBinding from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_cluster_role_binding",
            "read_cluster_role_binding",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the ClusterRoleBinding from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_cluster_role_binding",
            "delete_cluster_role_binding",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "ClusterRoleBinding":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ClusterRoleBindingList(_kuber_definitions.Collection):
    """
    ClusterRoleBindingList is a collection of
    ClusterRoleBindings
    """

    def __init__(
        self,
        items: typing.List["ClusterRoleBinding"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create ClusterRoleBindingList instance."""
        super(ClusterRoleBindingList, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1",
            kind="ClusterRoleBindingList",
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, ClusterRoleBinding),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["ClusterRoleBinding"]:
        """
        Items is a list of ClusterRoleBindings
        """
        return typing.cast(
            typing.List["ClusterRoleBinding"],
            self._properties.get("items"),
        )

    @items.setter
    def items(
        self, value: typing.Union[typing.List["ClusterRoleBinding"], typing.List[dict]]
    ):
        """
        Items is a list of ClusterRoleBindings
        """
        cleaned: typing.List[ClusterRoleBinding] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    ClusterRoleBinding,
                    ClusterRoleBinding().from_dict(item),
                )
            cleaned.append(typing.cast(ClusterRoleBinding, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "ClusterRoleBindingList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ClusterRoleList(_kuber_definitions.Collection):
    """
    ClusterRoleList is a collection of ClusterRoles
    """

    def __init__(
        self,
        items: typing.List["ClusterRole"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create ClusterRoleList instance."""
        super(ClusterRoleList, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="ClusterRoleList"
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, ClusterRole),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["ClusterRole"]:
        """
        Items is a list of ClusterRoles
        """
        return typing.cast(
            typing.List["ClusterRole"],
            self._properties.get("items"),
        )

    @items.setter
    def items(self, value: typing.Union[typing.List["ClusterRole"], typing.List[dict]]):
        """
        Items is a list of ClusterRoles
        """
        cleaned: typing.List[ClusterRole] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    ClusterRole,
                    ClusterRole().from_dict(item),
                )
            cleaned.append(typing.cast(ClusterRole, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "ClusterRoleList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class PolicyRule(_kuber_definitions.Definition):
    """
    PolicyRule holds information that describes a policy rule,
    but does not contain information about who the rule applies
    to or which namespace the rule applies to.
    """

    def __init__(
        self,
        api_groups: typing.List[str] = None,
        non_resource_urls: typing.List[str] = None,
        resource_names: typing.List[str] = None,
        resources: typing.List[str] = None,
        verbs: typing.List[str] = None,
    ):
        """Create PolicyRule instance."""
        super(PolicyRule, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="PolicyRule"
        )
        self._properties = {
            "apiGroups": api_groups if api_groups is not None else [],
            "nonResourceURLs": non_resource_urls
            if non_resource_urls is not None
            else [],
            "resourceNames": resource_names if resource_names is not None else [],
            "resources": resources if resources is not None else [],
            "verbs": verbs if verbs is not None else [],
        }
        self._types = {
            "apiGroups": (list, str),
            "nonResourceURLs": (list, str),
            "resourceNames": (list, str),
            "resources": (list, str),
            "verbs": (list, str),
        }

    @property
    def api_groups(self) -> typing.List[str]:
        """
        APIGroups is the name of the APIGroup that contains the
        resources.  If multiple API groups are specified, any action
        requested against one of the enumerated resources in any API
        group will be allowed.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("apiGroups"),
        )

    @api_groups.setter
    def api_groups(self, value: typing.List[str]):
        """
        APIGroups is the name of the APIGroup that contains the
        resources.  If multiple API groups are specified, any action
        requested against one of the enumerated resources in any API
        group will be allowed.
        """
        self._properties["apiGroups"] = value

    @property
    def non_resource_urls(self) -> typing.List[str]:
        """
        NonResourceURLs is a set of partial urls that a user should
        have access to.  *s are allowed, but only as the full, final
        step in the path Since non-resource URLs are not namespaced,
        this field is only applicable for ClusterRoles referenced
        from a ClusterRoleBinding. Rules can either apply to API
        resources (such as "pods" or "secrets") or non-resource URL
        paths (such as "/api"),  but not both.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("nonResourceURLs"),
        )

    @non_resource_urls.setter
    def non_resource_urls(self, value: typing.List[str]):
        """
        NonResourceURLs is a set of partial urls that a user should
        have access to.  *s are allowed, but only as the full, final
        step in the path Since non-resource URLs are not namespaced,
        this field is only applicable for ClusterRoles referenced
        from a ClusterRoleBinding. Rules can either apply to API
        resources (such as "pods" or "secrets") or non-resource URL
        paths (such as "/api"),  but not both.
        """
        self._properties["nonResourceURLs"] = value

    @property
    def resource_names(self) -> typing.List[str]:
        """
        ResourceNames is an optional white list of names that the
        rule applies to.  An empty set means that everything is
        allowed.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("resourceNames"),
        )

    @resource_names.setter
    def resource_names(self, value: typing.List[str]):
        """
        ResourceNames is an optional white list of names that the
        rule applies to.  An empty set means that everything is
        allowed.
        """
        self._properties["resourceNames"] = value

    @property
    def resources(self) -> typing.List[str]:
        """
        Resources is a list of resources this rule applies to.  '*'
        represents all resources in the specified apiGroups. '*/foo'
        represents the subresource 'foo' for all resources in the
        specified apiGroups.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("resources"),
        )

    @resources.setter
    def resources(self, value: typing.List[str]):
        """
        Resources is a list of resources this rule applies to.  '*'
        represents all resources in the specified apiGroups. '*/foo'
        represents the subresource 'foo' for all resources in the
        specified apiGroups.
        """
        self._properties["resources"] = value

    @property
    def verbs(self) -> typing.List[str]:
        """
        Verbs is a list of Verbs that apply to ALL the ResourceKinds
        and AttributeRestrictions contained in this rule.  VerbAll
        represents all kinds.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("verbs"),
        )

    @verbs.setter
    def verbs(self, value: typing.List[str]):
        """
        Verbs is a list of Verbs that apply to ALL the ResourceKinds
        and AttributeRestrictions contained in this rule.  VerbAll
        represents all kinds.
        """
        self._properties["verbs"] = value

    def __enter__(self) -> "PolicyRule":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class Role(_kuber_definitions.Resource):
    """
    Role is a namespaced, logical grouping of PolicyRules that
    can be referenced as a unit by a RoleBinding.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        rules: typing.List["PolicyRule"] = None,
    ):
        """Create Role instance."""
        super(Role, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="Role"
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "rules": rules if rules is not None else [],
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "rules": (list, PolicyRule),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def rules(self) -> typing.List["PolicyRule"]:
        """
        Rules holds all the PolicyRules for this Role
        """
        return typing.cast(
            typing.List["PolicyRule"],
            self._properties.get("rules"),
        )

    @rules.setter
    def rules(self, value: typing.Union[typing.List["PolicyRule"], typing.List[dict]]):
        """
        Rules holds all the PolicyRules for this Role
        """
        cleaned: typing.List[PolicyRule] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    PolicyRule,
                    PolicyRule().from_dict(item),
                )
            cleaned.append(typing.cast(PolicyRule, item))
        self._properties["rules"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the Role in the currently
        configured Kubernetes cluster.
        """
        names = ["create_namespaced_role", "create_role"]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the Role in the currently
        configured Kubernetes cluster.
        """
        names = ["replace_namespaced_role", "replace_role"]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the Role in the currently
        configured Kubernetes cluster.
        """
        names = ["patch_namespaced_role", "patch_role"]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the Role from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_role",
            "read_role",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the Role from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_role",
            "delete_role",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "Role":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class RoleBinding(_kuber_definitions.Resource):
    """
    RoleBinding references a role, but does not contain it.  It
    can reference a Role in the same namespace or a ClusterRole
    in the global namespace. It adds who information via
    Subjects and namespace information by which namespace it
    exists in.  RoleBindings in a given namespace only have
    effect in that namespace.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        role_ref: "RoleRef" = None,
        subjects: typing.List["Subject"] = None,
    ):
        """Create RoleBinding instance."""
        super(RoleBinding, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="RoleBinding"
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "roleRef": role_ref if role_ref is not None else RoleRef(),
            "subjects": subjects if subjects is not None else [],
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "roleRef": (RoleRef, None),
            "subjects": (list, Subject),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def role_ref(self) -> "RoleRef":
        """
        RoleRef can reference a Role in the current namespace or a
        ClusterRole in the global namespace. If the RoleRef cannot
        be resolved, the Authorizer must return an error.
        """
        return typing.cast(
            "RoleRef",
            self._properties.get("roleRef"),
        )

    @role_ref.setter
    def role_ref(self, value: typing.Union["RoleRef", dict]):
        """
        RoleRef can reference a Role in the current namespace or a
        ClusterRole in the global namespace. If the RoleRef cannot
        be resolved, the Authorizer must return an error.
        """
        if isinstance(value, dict):
            value = typing.cast(
                RoleRef,
                RoleRef().from_dict(value),
            )
        self._properties["roleRef"] = value

    @property
    def subjects(self) -> typing.List["Subject"]:
        """
        Subjects holds references to the objects the role applies
        to.
        """
        return typing.cast(
            typing.List["Subject"],
            self._properties.get("subjects"),
        )

    @subjects.setter
    def subjects(self, value: typing.Union[typing.List["Subject"], typing.List[dict]]):
        """
        Subjects holds references to the objects the role applies
        to.
        """
        cleaned: typing.List[Subject] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    Subject,
                    Subject().from_dict(item),
                )
            cleaned.append(typing.cast(Subject, item))
        self._properties["subjects"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the RoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = ["create_namespaced_role_binding", "create_role_binding"]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the RoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = ["replace_namespaced_role_binding", "replace_role_binding"]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the RoleBinding in the currently
        configured Kubernetes cluster.
        """
        names = ["patch_namespaced_role_binding", "patch_role_binding"]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the RoleBinding from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_role_binding",
            "read_role_binding",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the RoleBinding from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_role_binding",
            "delete_role_binding",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "RoleBinding":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class RoleBindingList(_kuber_definitions.Collection):
    """
    RoleBindingList is a collection of RoleBindings
    """

    def __init__(
        self,
        items: typing.List["RoleBinding"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create RoleBindingList instance."""
        super(RoleBindingList, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="RoleBindingList"
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, RoleBinding),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["RoleBinding"]:
        """
        Items is a list of RoleBindings
        """
        return typing.cast(
            typing.List["RoleBinding"],
            self._properties.get("items"),
        )

    @items.setter
    def items(self, value: typing.Union[typing.List["RoleBinding"], typing.List[dict]]):
        """
        Items is a list of RoleBindings
        """
        cleaned: typing.List[RoleBinding] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    RoleBinding,
                    RoleBinding().from_dict(item),
                )
            cleaned.append(typing.cast(RoleBinding, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "RoleBindingList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class RoleList(_kuber_definitions.Collection):
    """
    RoleList is a collection of Roles
    """

    def __init__(
        self,
        items: typing.List["Role"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create RoleList instance."""
        super(RoleList, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="RoleList"
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, Role),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["Role"]:
        """
        Items is a list of Roles
        """
        return typing.cast(
            typing.List["Role"],
            self._properties.get("items"),
        )

    @items.setter
    def items(self, value: typing.Union[typing.List["Role"], typing.List[dict]]):
        """
        Items is a list of Roles
        """
        cleaned: typing.List[Role] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    Role,
                    Role().from_dict(item),
                )
            cleaned.append(typing.cast(Role, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard object's metadata.
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard object's metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.RbacAuthorizationV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.RbacAuthorizationV1beta1Api(**kwargs)

    def __enter__(self) -> "RoleList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class RoleRef(_kuber_definitions.Definition):
    """
    RoleRef contains information that points to the role being
    used
    """

    def __init__(
        self,
        api_group: str = None,
        kind: str = None,
        name: str = None,
    ):
        """Create RoleRef instance."""
        super(RoleRef, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="RoleRef"
        )
        self._properties = {
            "apiGroup": api_group if api_group is not None else "",
            "kind": kind if kind is not None else "",
            "name": name if name is not None else "",
        }
        self._types = {
            "apiGroup": (str, None),
            "kind": (str, None),
            "name": (str, None),
        }

    @property
    def api_group(self) -> str:
        """
        APIGroup is the group for the resource being referenced
        """
        return typing.cast(
            str,
            self._properties.get("apiGroup"),
        )

    @api_group.setter
    def api_group(self, value: str):
        """
        APIGroup is the group for the resource being referenced
        """
        self._properties["apiGroup"] = value

    @property
    def kind(self) -> str:
        """
        Kind is the type of resource being referenced
        """
        return typing.cast(
            str,
            self._properties.get("kind"),
        )

    @kind.setter
    def kind(self, value: str):
        """
        Kind is the type of resource being referenced
        """
        self._properties["kind"] = value

    @property
    def name(self) -> str:
        """
        Name is the name of resource being referenced
        """
        return typing.cast(
            str,
            self._properties.get("name"),
        )

    @name.setter
    def name(self, value: str):
        """
        Name is the name of resource being referenced
        """
        self._properties["name"] = value

    def __enter__(self) -> "RoleRef":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class Subject(_kuber_definitions.Definition):
    """
    Subject contains a reference to the object or user
    identities a role binding applies to.  This can either hold
    a direct API object reference, or a value for non-objects
    such as user and group names.
    """

    def __init__(
        self,
        api_group: str = None,
        kind: str = None,
        name: str = None,
        namespace: str = None,
    ):
        """Create Subject instance."""
        super(Subject, self).__init__(
            api_version="rbac.authorization.k8s.io/v1beta1", kind="Subject"
        )
        self._properties = {
            "apiGroup": api_group if api_group is not None else "",
            "kind": kind if kind is not None else "",
            "name": name if name is not None else "",
            "namespace": namespace if namespace is not None else "",
        }
        self._types = {
            "apiGroup": (str, None),
            "kind": (str, None),
            "name": (str, None),
            "namespace": (str, None),
        }

    @property
    def api_group(self) -> str:
        """
        APIGroup holds the API group of the referenced subject.
        Defaults to "" for ServiceAccount subjects. Defaults to
        "rbac.authorization.k8s.io" for User and Group subjects.
        """
        return typing.cast(
            str,
            self._properties.get("apiGroup"),
        )

    @api_group.setter
    def api_group(self, value: str):
        """
        APIGroup holds the API group of the referenced subject.
        Defaults to "" for ServiceAccount subjects. Defaults to
        "rbac.authorization.k8s.io" for User and Group subjects.
        """
        self._properties["apiGroup"] = value

    @property
    def kind(self) -> str:
        """
        Kind of object being referenced. Values defined by this API
        group are "User", "Group", and "ServiceAccount". If the
        Authorizer does not recognized the kind value, the
        Authorizer should report an error.
        """
        return typing.cast(
            str,
            self._properties.get("kind"),
        )

    @kind.setter
    def kind(self, value: str):
        """
        Kind of object being referenced. Values defined by this API
        group are "User", "Group", and "ServiceAccount". If the
        Authorizer does not recognized the kind value, the
        Authorizer should report an error.
        """
        self._properties["kind"] = value

    @property
    def name(self) -> str:
        """
        Name of the object being referenced.
        """
        return typing.cast(
            str,
            self._properties.get("name"),
        )

    @name.setter
    def name(self, value: str):
        """
        Name of the object being referenced.
        """
        self._properties["name"] = value

    @property
    def namespace(self) -> str:
        """
        Namespace of the referenced object.  If the object kind is
        non-namespace, such as "User" or "Group", and this value is
        not empty the Authorizer should report an error.
        """
        return typing.cast(
            str,
            self._properties.get("namespace"),
        )

    @namespace.setter
    def namespace(self, value: str):
        """
        Namespace of the referenced object.  If the object kind is
        non-namespace, such as "User" or "Group", and this value is
        not empty the Authorizer should report an error.
        """
        self._properties["namespace"] = value

    def __enter__(self) -> "Subject":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



import typing

from kubernetes import client
from kuber import kube_api as _kube_api

from kuber import definitions as _kuber_definitions
from kuber.v1_18.meta_v1 import ListMeta
from kuber.v1_18.core_v1 import LoadBalancerStatus
from kuber.v1_18.meta_v1 import ObjectMeta
from kuber.v1_18.meta_v1 import Status
from kuber.v1_18.meta_v1 import StatusDetails
from kuber.v1_18.core_v1 import TypedLocalObjectReference


class HTTPIngressPath(_kuber_definitions.Definition):
    """
    HTTPIngressPath associates a path with a backend. Incoming
    urls matching the path are forwarded to the backend.
    """

    def __init__(
        self,
        backend: "IngressBackend" = None,
        path: str = None,
        path_type: str = None,
    ):
        """Create HTTPIngressPath instance."""
        super(HTTPIngressPath, self).__init__(
            api_version="networking/v1beta1", kind="HTTPIngressPath"
        )
        self._properties = {
            "backend": backend if backend is not None else IngressBackend(),
            "path": path if path is not None else "",
            "pathType": path_type if path_type is not None else "",
        }
        self._types = {
            "backend": (IngressBackend, None),
            "path": (str, None),
            "pathType": (str, None),
        }

    @property
    def backend(self) -> "IngressBackend":
        """
        Backend defines the referenced service endpoint to which the
        traffic will be forwarded to.
        """
        return typing.cast(
            "IngressBackend",
            self._properties.get("backend"),
        )

    @backend.setter
    def backend(self, value: typing.Union["IngressBackend", dict]):
        """
        Backend defines the referenced service endpoint to which the
        traffic will be forwarded to.
        """
        if isinstance(value, dict):
            value = typing.cast(
                IngressBackend,
                IngressBackend().from_dict(value),
            )
        self._properties["backend"] = value

    @property
    def path(self) -> str:
        """
        Path is matched against the path of an incoming request.
        Currently it can contain characters disallowed from the
        conventional "path" part of a URL as defined by RFC 3986.
        Paths must begin with a '/'. When unspecified, all paths
        from incoming requests are matched.
        """
        return typing.cast(
            str,
            self._properties.get("path"),
        )

    @path.setter
    def path(self, value: str):
        """
        Path is matched against the path of an incoming request.
        Currently it can contain characters disallowed from the
        conventional "path" part of a URL as defined by RFC 3986.
        Paths must begin with a '/'. When unspecified, all paths
        from incoming requests are matched.
        """
        self._properties["path"] = value

    @property
    def path_type(self) -> str:
        """
        PathType determines the interpretation of the Path matching.
        PathType can be one of the following values: * Exact:
        Matches the URL path exactly. * Prefix: Matches based on a
        URL path prefix split by '/'. Matching is
          done on a path element by element basis. A path element
        refers is the
          list of labels in the path split by the '/' separator. A
        request is a
          match for path p if every p is an element-wise prefix of p
        of the
          request path. Note that if the last element of the path is
        a substring
          of the last element in request path, it is not a match
        (e.g. /foo/bar
          matches /foo/bar/baz, but does not match /foo/barbaz).
        * ImplementationSpecific: Interpretation of the Path
        matching is up to
          the IngressClass. Implementations can treat this as a
        separate PathType
          or treat it identically to Prefix or Exact path types.
        Implementations are required to support all path types.
        Defaults to ImplementationSpecific.
        """
        return typing.cast(
            str,
            self._properties.get("pathType"),
        )

    @path_type.setter
    def path_type(self, value: str):
        """
        PathType determines the interpretation of the Path matching.
        PathType can be one of the following values: * Exact:
        Matches the URL path exactly. * Prefix: Matches based on a
        URL path prefix split by '/'. Matching is
          done on a path element by element basis. A path element
        refers is the
          list of labels in the path split by the '/' separator. A
        request is a
          match for path p if every p is an element-wise prefix of p
        of the
          request path. Note that if the last element of the path is
        a substring
          of the last element in request path, it is not a match
        (e.g. /foo/bar
          matches /foo/bar/baz, but does not match /foo/barbaz).
        * ImplementationSpecific: Interpretation of the Path
        matching is up to
          the IngressClass. Implementations can treat this as a
        separate PathType
          or treat it identically to Prefix or Exact path types.
        Implementations are required to support all path types.
        Defaults to ImplementationSpecific.
        """
        self._properties["pathType"] = value

    def __enter__(self) -> "HTTPIngressPath":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class HTTPIngressRuleValue(_kuber_definitions.Definition):
    """
    HTTPIngressRuleValue is a list of http selectors pointing to
    backends. In the example: http://<host>/<path>?<searchpart>
    -> backend where where parts of the url correspond to RFC
    3986, this resource will be used to match against everything
    after the last '/' and before the first '?' or '#'.
    """

    def __init__(
        self,
        paths: typing.List["HTTPIngressPath"] = None,
    ):
        """Create HTTPIngressRuleValue instance."""
        super(HTTPIngressRuleValue, self).__init__(
            api_version="networking/v1beta1", kind="HTTPIngressRuleValue"
        )
        self._properties = {
            "paths": paths if paths is not None else [],
        }
        self._types = {
            "paths": (list, HTTPIngressPath),
        }

    @property
    def paths(self) -> typing.List["HTTPIngressPath"]:
        """
        A collection of paths that map requests to backends.
        """
        return typing.cast(
            typing.List["HTTPIngressPath"],
            self._properties.get("paths"),
        )

    @paths.setter
    def paths(
        self, value: typing.Union[typing.List["HTTPIngressPath"], typing.List[dict]]
    ):
        """
        A collection of paths that map requests to backends.
        """
        cleaned: typing.List[HTTPIngressPath] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    HTTPIngressPath,
                    HTTPIngressPath().from_dict(item),
                )
            cleaned.append(typing.cast(HTTPIngressPath, item))
        self._properties["paths"] = cleaned

    def __enter__(self) -> "HTTPIngressRuleValue":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class Ingress(_kuber_definitions.Resource):
    """
    Ingress is a collection of rules that allow inbound
    connections to reach the endpoints defined by a backend. An
    Ingress can be configured to give services externally-
    reachable urls, load balance traffic, terminate SSL, offer
    name based virtual hosting etc.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        spec: "IngressSpec" = None,
        status: "IngressStatus" = None,
    ):
        """Create Ingress instance."""
        super(Ingress, self).__init__(api_version="networking/v1beta1", kind="Ingress")
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "spec": spec if spec is not None else IngressSpec(),
            "status": status if status is not None else IngressStatus(),
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "spec": (IngressSpec, None),
            "status": (IngressStatus, None),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def spec(self) -> "IngressSpec":
        """
        Spec is the desired state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        return typing.cast(
            "IngressSpec",
            self._properties.get("spec"),
        )

    @spec.setter
    def spec(self, value: typing.Union["IngressSpec", dict]):
        """
        Spec is the desired state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        if isinstance(value, dict):
            value = typing.cast(
                IngressSpec,
                IngressSpec().from_dict(value),
            )
        self._properties["spec"] = value

    @property
    def status(self) -> "IngressStatus":
        """
        Status is the current state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        return typing.cast(
            "IngressStatus",
            self._properties.get("status"),
        )

    @status.setter
    def status(self, value: typing.Union["IngressStatus", dict]):
        """
        Status is the current state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        if isinstance(value, dict):
            value = typing.cast(
                IngressStatus,
                IngressStatus().from_dict(value),
            )
        self._properties["status"] = value

    def create_resource(self, namespace: "str" = None) -> "IngressStatus":
        """
        Creates the Ingress in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the create is complete.
        """
        names = ["create_namespaced_ingress", "create_ingress"]

        response = _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

        output = IngressStatus()
        if response is not None:
            output.from_dict(_kube_api.to_kuber_dict(response.status))
        return output

    def replace_resource(self, namespace: "str" = None) -> "IngressStatus":
        """
        Replaces the Ingress in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the replace is complete.
        """
        names = ["replace_namespaced_ingress", "replace_ingress"]

        response = _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

        output = IngressStatus()
        if response is not None:
            output.from_dict(_kube_api.to_kuber_dict(response.status))
        return output

    def patch_resource(self, namespace: "str" = None) -> "IngressStatus":
        """
        Patches the Ingress in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the replace is complete.
        """
        names = ["patch_namespaced_ingress", "patch_ingress"]

        response = _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

        output = IngressStatus()
        if response is not None:
            output.from_dict(_kube_api.to_kuber_dict(response.status))
        return output

    def get_resource_status(self, namespace: "str" = None) -> "IngressStatus":
        """
        Returns status information about the given resource within the cluster.
        """
        names = ["read_namespaced_ingress", "read_ingress"]

        response = _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

        output = IngressStatus()
        if response is not None:
            output.from_dict(_kube_api.to_kuber_dict(response.status))
        return output

    def read_resource(self, namespace: str = None):
        """
        Reads the Ingress from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_ingress",
            "read_ingress",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the Ingress from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_ingress",
            "delete_ingress",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.NetworkingV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.NetworkingV1beta1Api(**kwargs)

    def __enter__(self) -> "Ingress":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressBackend(_kuber_definitions.Definition):
    """
    IngressBackend describes all endpoints for a given service
    and port.
    """

    def __init__(
        self,
        resource: "TypedLocalObjectReference" = None,
        service_name: str = None,
        service_port: typing.Union[str, int, None] = None,
    ):
        """Create IngressBackend instance."""
        super(IngressBackend, self).__init__(
            api_version="networking/v1beta1", kind="IngressBackend"
        )
        self._properties = {
            "resource": resource
            if resource is not None
            else TypedLocalObjectReference(),
            "serviceName": service_name if service_name is not None else "",
            "servicePort": service_port if service_port is not None else None,
        }
        self._types = {
            "resource": (TypedLocalObjectReference, None),
            "serviceName": (str, None),
            "servicePort": (int, None),
        }

    @property
    def resource(self) -> "TypedLocalObjectReference":
        """
        Resource is an ObjectRef to another Kubernetes resource in
        the namespace of the Ingress object. If resource is
        specified, serviceName and servicePort must not be
        specified.
        """
        return typing.cast(
            "TypedLocalObjectReference",
            self._properties.get("resource"),
        )

    @resource.setter
    def resource(self, value: typing.Union["TypedLocalObjectReference", dict]):
        """
        Resource is an ObjectRef to another Kubernetes resource in
        the namespace of the Ingress object. If resource is
        specified, serviceName and servicePort must not be
        specified.
        """
        if isinstance(value, dict):
            value = typing.cast(
                TypedLocalObjectReference,
                TypedLocalObjectReference().from_dict(value),
            )
        self._properties["resource"] = value

    @property
    def service_name(self) -> str:
        """
        Specifies the name of the referenced service.
        """
        return typing.cast(
            str,
            self._properties.get("serviceName"),
        )

    @service_name.setter
    def service_name(self, value: str):
        """
        Specifies the name of the referenced service.
        """
        self._properties["serviceName"] = value

    @property
    def service_port(self) -> typing.Optional[int]:
        """
        Specifies the port of the referenced service.
        """
        value = self._properties.get("servicePort")
        return int(value) if value is not None else None

    @service_port.setter
    def service_port(self, value: typing.Union[str, int, None]):
        """
        Specifies the port of the referenced service.
        """
        self._properties["servicePort"] = None if value is None else f"{value}"

    def __enter__(self) -> "IngressBackend":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressClass(_kuber_definitions.Resource):
    """
    IngressClass represents the class of the Ingress, referenced
    by the Ingress Spec. The `ingressclass.kubernetes.io/is-
    default-class` annotation can be used to indicate that an
    IngressClass should be considered default. When a single
    IngressClass resource has this annotation set to true, new
    Ingress resources without a class specified will be assigned
    this default class.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        spec: "IngressClassSpec" = None,
    ):
        """Create IngressClass instance."""
        super(IngressClass, self).__init__(
            api_version="networking/v1beta1", kind="IngressClass"
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "spec": spec if spec is not None else IngressClassSpec(),
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "spec": (IngressClassSpec, None),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def spec(self) -> "IngressClassSpec":
        """
        Spec is the desired state of the IngressClass. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        return typing.cast(
            "IngressClassSpec",
            self._properties.get("spec"),
        )

    @spec.setter
    def spec(self, value: typing.Union["IngressClassSpec", dict]):
        """
        Spec is the desired state of the IngressClass. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#spec-and-status
        """
        if isinstance(value, dict):
            value = typing.cast(
                IngressClassSpec,
                IngressClassSpec().from_dict(value),
            )
        self._properties["spec"] = value

    def create_resource(self, namespace: "str" = None):
        """
        Creates the IngressClass in the currently
        configured Kubernetes cluster.
        """
        names = ["create_namespaced_ingress_class", "create_ingress_class"]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the IngressClass in the currently
        configured Kubernetes cluster.
        """
        names = ["replace_namespaced_ingress_class", "replace_ingress_class"]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the IngressClass in the currently
        configured Kubernetes cluster.
        """
        names = ["patch_namespaced_ingress_class", "patch_ingress_class"]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the IngressClass from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_ingress_class",
            "read_ingress_class",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the IngressClass from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_ingress_class",
            "delete_ingress_class",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.NetworkingV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.NetworkingV1beta1Api(**kwargs)

    def __enter__(self) -> "IngressClass":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressClassList(_kuber_definitions.Collection):
    """
    IngressClassList is a collection of IngressClasses.
    """

    def __init__(
        self,
        items: typing.List["IngressClass"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create IngressClassList instance."""
        super(IngressClassList, self).__init__(
            api_version="networking/v1beta1", kind="IngressClassList"
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, IngressClass),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["IngressClass"]:
        """
        Items is the list of IngressClasses.
        """
        return typing.cast(
            typing.List["IngressClass"],
            self._properties.get("items"),
        )

    @items.setter
    def items(
        self, value: typing.Union[typing.List["IngressClass"], typing.List[dict]]
    ):
        """
        Items is the list of IngressClasses.
        """
        cleaned: typing.List[IngressClass] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    IngressClass,
                    IngressClass().from_dict(item),
                )
            cleaned.append(typing.cast(IngressClass, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard list metadata.
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard list metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.NetworkingV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.NetworkingV1beta1Api(**kwargs)

    def __enter__(self) -> "IngressClassList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressClassSpec(_kuber_definitions.Definition):
    """
    IngressClassSpec provides information about the class of an
    Ingress.
    """

    def __init__(
        self,
        controller: str = None,
        parameters: "TypedLocalObjectReference" = None,
    ):
        """Create IngressClassSpec instance."""
        super(IngressClassSpec, self).__init__(
            api_version="networking/v1beta1", kind="IngressClassSpec"
        )
        self._properties = {
            "controller": controller if controller is not None else "",
            "parameters": parameters
            if parameters is not None
            else TypedLocalObjectReference(),
        }
        self._types = {
            "controller": (str, None),
            "parameters": (TypedLocalObjectReference, None),
        }

    @property
    def controller(self) -> str:
        """
        Controller refers to the name of the controller that should
        handle this class. This allows for different "flavors" that
        are controlled by the same controller. For example, you may
        have different Parameters for the same implementing
        controller. This should be specified as a domain-prefixed
        path no more than 250 characters in length, e.g.
        "acme.io/ingress-controller". This field is immutable.
        """
        return typing.cast(
            str,
            self._properties.get("controller"),
        )

    @controller.setter
    def controller(self, value: str):
        """
        Controller refers to the name of the controller that should
        handle this class. This allows for different "flavors" that
        are controlled by the same controller. For example, you may
        have different Parameters for the same implementing
        controller. This should be specified as a domain-prefixed
        path no more than 250 characters in length, e.g.
        "acme.io/ingress-controller". This field is immutable.
        """
        self._properties["controller"] = value

    @property
    def parameters(self) -> "TypedLocalObjectReference":
        """
        Parameters is a link to a custom resource containing
        additional configuration for the controller. This is
        optional if the controller does not require extra
        parameters.
        """
        return typing.cast(
            "TypedLocalObjectReference",
            self._properties.get("parameters"),
        )

    @parameters.setter
    def parameters(self, value: typing.Union["TypedLocalObjectReference", dict]):
        """
        Parameters is a link to a custom resource containing
        additional configuration for the controller. This is
        optional if the controller does not require extra
        parameters.
        """
        if isinstance(value, dict):
            value = typing.cast(
                TypedLocalObjectReference,
                TypedLocalObjectReference().from_dict(value),
            )
        self._properties["parameters"] = value

    def __enter__(self) -> "IngressClassSpec":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressList(_kuber_definitions.Collection):
    """
    IngressList is a collection of Ingress.
    """

    def __init__(
        self,
        items: typing.List["Ingress"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create IngressList instance."""
        super(IngressList, self).__init__(
            api_version="networking/v1beta1", kind="IngressList"
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, Ingress),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["Ingress"]:
        """
        Items is the list of Ingress.
        """
        return typing.cast(
            typing.List["Ingress"],
            self._properties.get("items"),
        )

    @items.setter
    def items(self, value: typing.Union[typing.List["Ingress"], typing.List[dict]]):
        """
        Items is the list of Ingress.
        """
        cleaned: typing.List[Ingress] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    Ingress,
                    Ingress().from_dict(item),
                )
            cleaned.append(typing.cast(Ingress, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.NetworkingV1beta1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.NetworkingV1beta1Api(**kwargs)

    def __enter__(self) -> "IngressList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressRule(_kuber_definitions.Definition):
    """
    IngressRule represents the rules mapping the paths under a
    specified host to the related backend services. Incoming
    requests are first evaluated for a host match, then routed
    to the backend associated with the matching
    IngressRuleValue.
    """

    def __init__(
        self,
        host: str = None,
        http: "HTTPIngressRuleValue" = None,
    ):
        """Create IngressRule instance."""
        super(IngressRule, self).__init__(
            api_version="networking/v1beta1", kind="IngressRule"
        )
        self._properties = {
            "host": host if host is not None else "",
            "http": http if http is not None else HTTPIngressRuleValue(),
        }
        self._types = {
            "host": (str, None),
            "http": (HTTPIngressRuleValue, None),
        }

    @property
    def host(self) -> str:
        """
        Host is the fully qualified domain name of a network host,
        as defined by RFC 3986. Note the following deviations from
        the "host" part of the URI as defined in RFC 3986: 1. IPs
        are not allowed. Currently an IngressRuleValue can only
        apply to
           the IP in the Spec of the parent Ingress.
        2. The `:` delimiter is not respected because ports are not
        allowed.
                  Currently the port of an Ingress is implicitly :80 for
        http and
                  :443 for https.
        Both these may change in the future. Incoming requests are
        matched against the host before the IngressRuleValue. If the
        host is unspecified, the Ingress routes all traffic based on
        the specified IngressRuleValue.

        Host can be "precise" which is a domain name without the
        terminating dot of a network host (e.g. "foo.bar.com") or
        "wildcard", which is a domain name prefixed with a single
        wildcard label (e.g. "*.foo.com"). The wildcard character
        '*' must appear by itself as the first DNS label and matches
        only a single label. You cannot have a wildcard label by
        itself (e.g. Host == "*"). Requests will be matched against
        the Host field in the following way: 1. If Host is precise,
        the request matches this rule if the http host header is
        equal to Host. 2. If Host is a wildcard, then the request
        matches this rule if the http host header is to equal to the
        suffix (removing the first label) of the wildcard rule.
        """
        return typing.cast(
            str,
            self._properties.get("host"),
        )

    @host.setter
    def host(self, value: str):
        """
        Host is the fully qualified domain name of a network host,
        as defined by RFC 3986. Note the following deviations from
        the "host" part of the URI as defined in RFC 3986: 1. IPs
        are not allowed. Currently an IngressRuleValue can only
        apply to
           the IP in the Spec of the parent Ingress.
        2. The `:` delimiter is not respected because ports are not
        allowed.
                  Currently the port of an Ingress is implicitly :80 for
        http and
                  :443 for https.
        Both these may change in the future. Incoming requests are
        matched against the host before the IngressRuleValue. If the
        host is unspecified, the Ingress routes all traffic based on
        the specified IngressRuleValue.

        Host can be "precise" which is a domain name without the
        terminating dot of a network host (e.g. "foo.bar.com") or
        "wildcard", which is a domain name prefixed with a single
        wildcard label (e.g. "*.foo.com"). The wildcard character
        '*' must appear by itself as the first DNS label and matches
        only a single label. You cannot have a wildcard label by
        itself (e.g. Host == "*"). Requests will be matched against
        the Host field in the following way: 1. If Host is precise,
        the request matches this rule if the http host header is
        equal to Host. 2. If Host is a wildcard, then the request
        matches this rule if the http host header is to equal to the
        suffix (removing the first label) of the wildcard rule.
        """
        self._properties["host"] = value

    @property
    def http(self) -> "HTTPIngressRuleValue":
        """"""
        return typing.cast(
            "HTTPIngressRuleValue",
            self._properties.get("http"),
        )

    @http.setter
    def http(self, value: typing.Union["HTTPIngressRuleValue", dict]):
        """"""
        if isinstance(value, dict):
            value = typing.cast(
                HTTPIngressRuleValue,
                HTTPIngressRuleValue().from_dict(value),
            )
        self._properties["http"] = value

    def __enter__(self) -> "IngressRule":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressSpec(_kuber_definitions.Definition):
    """
    IngressSpec describes the Ingress the user wishes to exist.
    """

    def __init__(
        self,
        backend: "IngressBackend" = None,
        ingress_class_name: str = None,
        rules: typing.List["IngressRule"] = None,
        tls: typing.List["IngressTLS"] = None,
    ):
        """Create IngressSpec instance."""
        super(IngressSpec, self).__init__(
            api_version="networking/v1beta1", kind="IngressSpec"
        )
        self._properties = {
            "backend": backend if backend is not None else IngressBackend(),
            "ingressClassName": ingress_class_name
            if ingress_class_name is not None
            else "",
            "rules": rules if rules is not None else [],
            "tls": tls if tls is not None else [],
        }
        self._types = {
            "backend": (IngressBackend, None),
            "ingressClassName": (str, None),
            "rules": (list, IngressRule),
            "tls": (list, IngressTLS),
        }

    @property
    def backend(self) -> "IngressBackend":
        """
        A default backend capable of servicing requests that don't
        match any rule. At least one of 'backend' or 'rules' must be
        specified. This field is optional to allow the loadbalancer
        controller or defaulting logic to specify a global default.
        """
        return typing.cast(
            "IngressBackend",
            self._properties.get("backend"),
        )

    @backend.setter
    def backend(self, value: typing.Union["IngressBackend", dict]):
        """
        A default backend capable of servicing requests that don't
        match any rule. At least one of 'backend' or 'rules' must be
        specified. This field is optional to allow the loadbalancer
        controller or defaulting logic to specify a global default.
        """
        if isinstance(value, dict):
            value = typing.cast(
                IngressBackend,
                IngressBackend().from_dict(value),
            )
        self._properties["backend"] = value

    @property
    def ingress_class_name(self) -> str:
        """
        IngressClassName is the name of the IngressClass cluster
        resource. The associated IngressClass defines which
        controller will implement the resource. This replaces the
        deprecated `kubernetes.io/ingress.class` annotation. For
        backwards compatibility, when that annotation is set, it
        must be given precedence over this field. The controller may
        emit a warning if the field and annotation have different
        values. Implementations of this API should ignore Ingresses
        without a class specified. An IngressClass resource may be
        marked as default, which can be used to set a default value
        for this field. For more information, refer to the
        IngressClass documentation.
        """
        return typing.cast(
            str,
            self._properties.get("ingressClassName"),
        )

    @ingress_class_name.setter
    def ingress_class_name(self, value: str):
        """
        IngressClassName is the name of the IngressClass cluster
        resource. The associated IngressClass defines which
        controller will implement the resource. This replaces the
        deprecated `kubernetes.io/ingress.class` annotation. For
        backwards compatibility, when that annotation is set, it
        must be given precedence over this field. The controller may
        emit a warning if the field and annotation have different
        values. Implementations of this API should ignore Ingresses
        without a class specified. An IngressClass resource may be
        marked as default, which can be used to set a default value
        for this field. For more information, refer to the
        IngressClass documentation.
        """
        self._properties["ingressClassName"] = value

    @property
    def rules(self) -> typing.List["IngressRule"]:
        """
        A list of host rules used to configure the Ingress. If
        unspecified, or no rule matches, all traffic is sent to the
        default backend.
        """
        return typing.cast(
            typing.List["IngressRule"],
            self._properties.get("rules"),
        )

    @rules.setter
    def rules(self, value: typing.Union[typing.List["IngressRule"], typing.List[dict]]):
        """
        A list of host rules used to configure the Ingress. If
        unspecified, or no rule matches, all traffic is sent to the
        default backend.
        """
        cleaned: typing.List[IngressRule] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    IngressRule,
                    IngressRule().from_dict(item),
                )
            cleaned.append(typing.cast(IngressRule, item))
        self._properties["rules"] = cleaned

    @property
    def tls(self) -> typing.List["IngressTLS"]:
        """
        TLS configuration. Currently the Ingress only supports a
        single TLS port, 443. If multiple members of this list
        specify different hosts, they will be multiplexed on the
        same port according to the hostname specified through the
        SNI TLS extension, if the ingress controller fulfilling the
        ingress supports SNI.
        """
        return typing.cast(
            typing.List["IngressTLS"],
            self._properties.get("tls"),
        )

    @tls.setter
    def tls(self, value: typing.Union[typing.List["IngressTLS"], typing.List[dict]]):
        """
        TLS configuration. Currently the Ingress only supports a
        single TLS port, 443. If multiple members of this list
        specify different hosts, they will be multiplexed on the
        same port according to the hostname specified through the
        SNI TLS extension, if the ingress controller fulfilling the
        ingress supports SNI.
        """
        cleaned: typing.List[IngressTLS] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    IngressTLS,
                    IngressTLS().from_dict(item),
                )
            cleaned.append(typing.cast(IngressTLS, item))
        self._properties["tls"] = cleaned

    def __enter__(self) -> "IngressSpec":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressStatus(_kuber_definitions.Definition):
    """
    IngressStatus describe the current state of the Ingress.
    """

    def __init__(
        self,
        load_balancer: "LoadBalancerStatus" = None,
    ):
        """Create IngressStatus instance."""
        super(IngressStatus, self).__init__(
            api_version="networking/v1beta1", kind="IngressStatus"
        )
        self._properties = {
            "loadBalancer": load_balancer
            if load_balancer is not None
            else LoadBalancerStatus(),
        }
        self._types = {
            "loadBalancer": (LoadBalancerStatus, None),
        }

    @property
    def load_balancer(self) -> "LoadBalancerStatus":
        """
        LoadBalancer contains the current status of the load-
        balancer.
        """
        return typing.cast(
            "LoadBalancerStatus",
            self._properties.get("loadBalancer"),
        )

    @load_balancer.setter
    def load_balancer(self, value: typing.Union["LoadBalancerStatus", dict]):
        """
        LoadBalancer contains the current status of the load-
        balancer.
        """
        if isinstance(value, dict):
            value = typing.cast(
                LoadBalancerStatus,
                LoadBalancerStatus().from_dict(value),
            )
        self._properties["loadBalancer"] = value

    def __enter__(self) -> "IngressStatus":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class IngressTLS(_kuber_definitions.Definition):
    """
    IngressTLS describes the transport layer security associated
    with an Ingress.
    """

    def __init__(
        self,
        hosts: typing.List[str] = None,
        secret_name: str = None,
    ):
        """Create IngressTLS instance."""
        super(IngressTLS, self).__init__(
            api_version="networking/v1beta1", kind="IngressTLS"
        )
        self._properties = {
            "hosts": hosts if hosts is not None else [],
            "secretName": secret_name if secret_name is not None else "",
        }
        self._types = {
            "hosts": (list, str),
            "secretName": (str, None),
        }

    @property
    def hosts(self) -> typing.List[str]:
        """
        Hosts are a list of hosts included in the TLS certificate.
        The values in this list must match the name/s used in the
        tlsSecret. Defaults to the wildcard host setting for the
        loadbalancer controller fulfilling this Ingress, if left
        unspecified.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("hosts"),
        )

    @hosts.setter
    def hosts(self, value: typing.List[str]):
        """
        Hosts are a list of hosts included in the TLS certificate.
        The values in this list must match the name/s used in the
        tlsSecret. Defaults to the wildcard host setting for the
        loadbalancer controller fulfilling this Ingress, if left
        unspecified.
        """
        self._properties["hosts"] = value

    @property
    def secret_name(self) -> str:
        """
        SecretName is the name of the secret used to terminate TLS
        traffic on port 443. Field is left optional to allow TLS
        routing based on SNI hostname alone. If the SNI host in a
        listener conflicts with the "Host" header field used by an
        IngressRule, the SNI host is used for termination and value
        of the Host header is used for routing.
        """
        return typing.cast(
            str,
            self._properties.get("secretName"),
        )

    @secret_name.setter
    def secret_name(self, value: str):
        """
        SecretName is the name of the secret used to terminate TLS
        traffic on port 443. Field is left optional to allow TLS
        routing based on SNI hostname alone. If the SNI host in a
        listener conflicts with the "Host" header field used by an
        IngressRule, the SNI host is used for termination and value
        of the Host header is used for routing.
        """
        self._properties["secretName"] = value

    def __enter__(self) -> "IngressTLS":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



import typing

from kubernetes import client
from kuber import kube_api as _kube_api

from kuber import definitions as _kuber_definitions
from kuber.v1_19.meta_v1 import LabelSelector
from kuber.v1_19.meta_v1 import ListMeta
from kuber.v1_19.meta_v1 import ObjectMeta


class MutatingWebhook(_kuber_definitions.Definition):
    """
    MutatingWebhook describes an admission webhook and the
    resources and operations it applies to.
    """

    def __init__(
        self,
        admission_review_versions: typing.List[str] = None,
        client_config: "WebhookClientConfig" = None,
        failure_policy: str = None,
        match_policy: str = None,
        name: str = None,
        namespace_selector: "LabelSelector" = None,
        object_selector: "LabelSelector" = None,
        reinvocation_policy: str = None,
        rules: typing.List["RuleWithOperations"] = None,
        side_effects: str = None,
        timeout_seconds: int = None,
    ):
        """Create MutatingWebhook instance."""
        super(MutatingWebhook, self).__init__(
            api_version="admissionregistration/v1", kind="MutatingWebhook"
        )
        self._properties = {
            "admissionReviewVersions": admission_review_versions
            if admission_review_versions is not None
            else [],
            "clientConfig": client_config
            if client_config is not None
            else WebhookClientConfig(),
            "failurePolicy": failure_policy if failure_policy is not None else "",
            "matchPolicy": match_policy if match_policy is not None else "",
            "name": name if name is not None else "",
            "namespaceSelector": namespace_selector
            if namespace_selector is not None
            else LabelSelector(),
            "objectSelector": object_selector
            if object_selector is not None
            else LabelSelector(),
            "reinvocationPolicy": reinvocation_policy
            if reinvocation_policy is not None
            else "",
            "rules": rules if rules is not None else [],
            "sideEffects": side_effects if side_effects is not None else "",
            "timeoutSeconds": timeout_seconds if timeout_seconds is not None else None,
        }
        self._types = {
            "admissionReviewVersions": (list, str),
            "clientConfig": (WebhookClientConfig, None),
            "failurePolicy": (str, None),
            "matchPolicy": (str, None),
            "name": (str, None),
            "namespaceSelector": (LabelSelector, None),
            "objectSelector": (LabelSelector, None),
            "reinvocationPolicy": (str, None),
            "rules": (list, RuleWithOperations),
            "sideEffects": (str, None),
            "timeoutSeconds": (int, None),
        }

    @property
    def admission_review_versions(self) -> typing.List[str]:
        """
        AdmissionReviewVersions is an ordered list of preferred
        `AdmissionReview` versions the Webhook expects. API server
        will try to use first version in the list which it supports.
        If none of the versions specified in this list supported by
        API server, validation will fail for this object. If a
        persisted webhook configuration specifies allowed versions
        and does not include any versions known to the API Server,
        calls to the webhook will fail and be subject to the failure
        policy.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("admissionReviewVersions"),
        )

    @admission_review_versions.setter
    def admission_review_versions(self, value: typing.List[str]):
        """
        AdmissionReviewVersions is an ordered list of preferred
        `AdmissionReview` versions the Webhook expects. API server
        will try to use first version in the list which it supports.
        If none of the versions specified in this list supported by
        API server, validation will fail for this object. If a
        persisted webhook configuration specifies allowed versions
        and does not include any versions known to the API Server,
        calls to the webhook will fail and be subject to the failure
        policy.
        """
        self._properties["admissionReviewVersions"] = value

    @property
    def client_config(self) -> "WebhookClientConfig":
        """
        ClientConfig defines how to communicate with the hook.
        Required
        """
        return typing.cast(
            "WebhookClientConfig",
            self._properties.get("clientConfig"),
        )

    @client_config.setter
    def client_config(self, value: typing.Union["WebhookClientConfig", dict]):
        """
        ClientConfig defines how to communicate with the hook.
        Required
        """
        if isinstance(value, dict):
            value = typing.cast(
                WebhookClientConfig,
                WebhookClientConfig().from_dict(value),
            )
        self._properties["clientConfig"] = value

    @property
    def failure_policy(self) -> str:
        """
        FailurePolicy defines how unrecognized errors from the
        admission endpoint are handled - allowed values are Ignore
        or Fail. Defaults to Fail.
        """
        return typing.cast(
            str,
            self._properties.get("failurePolicy"),
        )

    @failure_policy.setter
    def failure_policy(self, value: str):
        """
        FailurePolicy defines how unrecognized errors from the
        admission endpoint are handled - allowed values are Ignore
        or Fail. Defaults to Fail.
        """
        self._properties["failurePolicy"] = value

    @property
    def match_policy(self) -> str:
        """
        matchPolicy defines how the "rules" list is used to match
        incoming requests. Allowed values are "Exact" or
        "Equivalent".

        - Exact: match a request only if it exactly matches a
        specified rule. For example, if deployments can be modified
        via apps/v1, apps/v1beta1, and extensions/v1beta1, but
        "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request
        to apps/v1beta1 or extensions/v1beta1 would not be sent to
        the webhook.

        - Equivalent: match a request if modifies a resource listed
        in rules, even via another API group or version. For
        example, if deployments can be modified via apps/v1,
        apps/v1beta1, and extensions/v1beta1, and "rules" only
        included `apiGroups:["apps"], apiVersions:["v1"], resources:
        ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would be converted to apps/v1 and sent to
        the webhook.

        Defaults to "Equivalent"
        """
        return typing.cast(
            str,
            self._properties.get("matchPolicy"),
        )

    @match_policy.setter
    def match_policy(self, value: str):
        """
        matchPolicy defines how the "rules" list is used to match
        incoming requests. Allowed values are "Exact" or
        "Equivalent".

        - Exact: match a request only if it exactly matches a
        specified rule. For example, if deployments can be modified
        via apps/v1, apps/v1beta1, and extensions/v1beta1, but
        "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request
        to apps/v1beta1 or extensions/v1beta1 would not be sent to
        the webhook.

        - Equivalent: match a request if modifies a resource listed
        in rules, even via another API group or version. For
        example, if deployments can be modified via apps/v1,
        apps/v1beta1, and extensions/v1beta1, and "rules" only
        included `apiGroups:["apps"], apiVersions:["v1"], resources:
        ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would be converted to apps/v1 and sent to
        the webhook.

        Defaults to "Equivalent"
        """
        self._properties["matchPolicy"] = value

    @property
    def name(self) -> str:
        """
        The name of the admission webhook. Name should be fully
        qualified, e.g., imagepolicy.kubernetes.io, where
        "imagepolicy" is the name of the webhook, and kubernetes.io
        is the name of the organization. Required.
        """
        return typing.cast(
            str,
            self._properties.get("name"),
        )

    @name.setter
    def name(self, value: str):
        """
        The name of the admission webhook. Name should be fully
        qualified, e.g., imagepolicy.kubernetes.io, where
        "imagepolicy" is the name of the webhook, and kubernetes.io
        is the name of the organization. Required.
        """
        self._properties["name"] = value

    @property
    def namespace_selector(self) -> "LabelSelector":
        """
        NamespaceSelector decides whether to run the webhook on an
        object based on whether the namespace for that object
        matches the selector. If the object itself is a namespace,
        the matching is performed on object.metadata.labels. If the
        object is another cluster scoped resource, it never skips
        the webhook.

        For example, to run the webhook on any objects whose
        namespace is not associated with "runlevel" of "0" or "1";
        you will set the selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }

        If instead you want to only run the webhook on any objects
        whose namespace is associated with the "environment" of
        "prod" or "staging"; you will set the selector as follows:
        "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }

        See https://kubernetes.io/docs/concepts/overview/working-
        with-objects/labels/ for more examples of label selectors.

        Default to the empty LabelSelector, which matches
        everything.
        """
        return typing.cast(
            "LabelSelector",
            self._properties.get("namespaceSelector"),
        )

    @namespace_selector.setter
    def namespace_selector(self, value: typing.Union["LabelSelector", dict]):
        """
        NamespaceSelector decides whether to run the webhook on an
        object based on whether the namespace for that object
        matches the selector. If the object itself is a namespace,
        the matching is performed on object.metadata.labels. If the
        object is another cluster scoped resource, it never skips
        the webhook.

        For example, to run the webhook on any objects whose
        namespace is not associated with "runlevel" of "0" or "1";
        you will set the selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }

        If instead you want to only run the webhook on any objects
        whose namespace is associated with the "environment" of
        "prod" or "staging"; you will set the selector as follows:
        "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }

        See https://kubernetes.io/docs/concepts/overview/working-
        with-objects/labels/ for more examples of label selectors.

        Default to the empty LabelSelector, which matches
        everything.
        """
        if isinstance(value, dict):
            value = typing.cast(
                LabelSelector,
                LabelSelector().from_dict(value),
            )
        self._properties["namespaceSelector"] = value

    @property
    def object_selector(self) -> "LabelSelector":
        """
        ObjectSelector decides whether to run the webhook based on
        if the object has matching labels. objectSelector is
        evaluated against both the oldObject and newObject that
        would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject
        in the case of create, or newObject in the case of delete)
        or an object that cannot have labels (like a
        DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the
        webhook is opt-in, because end users may skip the admission
        webhook by setting the labels. Default to the empty
        LabelSelector, which matches everything.
        """
        return typing.cast(
            "LabelSelector",
            self._properties.get("objectSelector"),
        )

    @object_selector.setter
    def object_selector(self, value: typing.Union["LabelSelector", dict]):
        """
        ObjectSelector decides whether to run the webhook based on
        if the object has matching labels. objectSelector is
        evaluated against both the oldObject and newObject that
        would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject
        in the case of create, or newObject in the case of delete)
        or an object that cannot have labels (like a
        DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the
        webhook is opt-in, because end users may skip the admission
        webhook by setting the labels. Default to the empty
        LabelSelector, which matches everything.
        """
        if isinstance(value, dict):
            value = typing.cast(
                LabelSelector,
                LabelSelector().from_dict(value),
            )
        self._properties["objectSelector"] = value

    @property
    def reinvocation_policy(self) -> str:
        """
        reinvocationPolicy indicates whether this webhook should be
        called multiple times as part of a single admission
        evaluation. Allowed values are "Never" and "IfNeeded".

        Never: the webhook will not be called more than once in a
        single admission evaluation.

        IfNeeded: the webhook will be called at least one additional
        time as part of the admission evaluation if the object being
        admitted is modified by other admission plugins after the
        initial webhook call. Webhooks that specify this option
        *must* be idempotent, able to process objects they
        previously admitted. Note: * the number of additional
        invocations is not guaranteed to be exactly one. * if
        additional invocations result in further modifications to
        the object, webhooks are not guaranteed to be invoked again.
        * webhooks that use this option may be reordered to minimize
        the number of additional invocations. * to validate an
        object after all mutations are guaranteed complete, use a
        validating admission webhook instead.

        Defaults to "Never".
        """
        return typing.cast(
            str,
            self._properties.get("reinvocationPolicy"),
        )

    @reinvocation_policy.setter
    def reinvocation_policy(self, value: str):
        """
        reinvocationPolicy indicates whether this webhook should be
        called multiple times as part of a single admission
        evaluation. Allowed values are "Never" and "IfNeeded".

        Never: the webhook will not be called more than once in a
        single admission evaluation.

        IfNeeded: the webhook will be called at least one additional
        time as part of the admission evaluation if the object being
        admitted is modified by other admission plugins after the
        initial webhook call. Webhooks that specify this option
        *must* be idempotent, able to process objects they
        previously admitted. Note: * the number of additional
        invocations is not guaranteed to be exactly one. * if
        additional invocations result in further modifications to
        the object, webhooks are not guaranteed to be invoked again.
        * webhooks that use this option may be reordered to minimize
        the number of additional invocations. * to validate an
        object after all mutations are guaranteed complete, use a
        validating admission webhook instead.

        Defaults to "Never".
        """
        self._properties["reinvocationPolicy"] = value

    @property
    def rules(self) -> typing.List["RuleWithOperations"]:
        """
        Rules describes what operations on what
        resources/subresources the webhook cares about. The webhook
        cares about an operation if it matches _any_ Rule. However,
        in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a
        state which cannot be recovered from without completely
        disabling the plugin, ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks are never called on admission
        requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
        """
        return typing.cast(
            typing.List["RuleWithOperations"],
            self._properties.get("rules"),
        )

    @rules.setter
    def rules(
        self, value: typing.Union[typing.List["RuleWithOperations"], typing.List[dict]]
    ):
        """
        Rules describes what operations on what
        resources/subresources the webhook cares about. The webhook
        cares about an operation if it matches _any_ Rule. However,
        in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a
        state which cannot be recovered from without completely
        disabling the plugin, ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks are never called on admission
        requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
        """
        cleaned: typing.List[RuleWithOperations] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    RuleWithOperations,
                    RuleWithOperations().from_dict(item),
                )
            cleaned.append(typing.cast(RuleWithOperations, item))
        self._properties["rules"] = cleaned

    @property
    def side_effects(self) -> str:
        """
        SideEffects states whether this webhook has side effects.
        Acceptable values are: None, NoneOnDryRun (webhooks created
        via v1beta1 may also specify Some or Unknown). Webhooks with
        side effects MUST implement a reconciliation system, since a
        request may be rejected by a future step in the admission
        change and the side effects therefore need to be undone.
        Requests with the dryRun attribute will be auto-rejected if
        they match a webhook with sideEffects == Unknown or Some.
        """
        return typing.cast(
            str,
            self._properties.get("sideEffects"),
        )

    @side_effects.setter
    def side_effects(self, value: str):
        """
        SideEffects states whether this webhook has side effects.
        Acceptable values are: None, NoneOnDryRun (webhooks created
        via v1beta1 may also specify Some or Unknown). Webhooks with
        side effects MUST implement a reconciliation system, since a
        request may be rejected by a future step in the admission
        change and the side effects therefore need to be undone.
        Requests with the dryRun attribute will be auto-rejected if
        they match a webhook with sideEffects == Unknown or Some.
        """
        self._properties["sideEffects"] = value

    @property
    def timeout_seconds(self) -> int:
        """
        TimeoutSeconds specifies the timeout for this webhook. After
        the timeout passes, the webhook call will be ignored or the
        API call will fail based on the failure policy. The timeout
        value must be between 1 and 30 seconds. Default to 10
        seconds.
        """
        return typing.cast(
            int,
            self._properties.get("timeoutSeconds"),
        )

    @timeout_seconds.setter
    def timeout_seconds(self, value: int):
        """
        TimeoutSeconds specifies the timeout for this webhook. After
        the timeout passes, the webhook call will be ignored or the
        API call will fail based on the failure policy. The timeout
        value must be between 1 and 30 seconds. Default to 10
        seconds.
        """
        self._properties["timeoutSeconds"] = value

    def __enter__(self) -> "MutatingWebhook":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class MutatingWebhookConfiguration(_kuber_definitions.Resource):
    """
    MutatingWebhookConfiguration describes the configuration of
    and admission webhook that accept or reject and may change
    the object.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        webhooks: typing.List["MutatingWebhook"] = None,
    ):
        """Create MutatingWebhookConfiguration instance."""
        super(MutatingWebhookConfiguration, self).__init__(
            api_version="admissionregistration/v1", kind="MutatingWebhookConfiguration"
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "webhooks": webhooks if webhooks is not None else [],
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "webhooks": (list, MutatingWebhook),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def webhooks(self) -> typing.List["MutatingWebhook"]:
        """
        Webhooks is a list of webhooks and the affected resources
        and operations.
        """
        return typing.cast(
            typing.List["MutatingWebhook"],
            self._properties.get("webhooks"),
        )

    @webhooks.setter
    def webhooks(
        self, value: typing.Union[typing.List["MutatingWebhook"], typing.List[dict]]
    ):
        """
        Webhooks is a list of webhooks and the affected resources
        and operations.
        """
        cleaned: typing.List[MutatingWebhook] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    MutatingWebhook,
                    MutatingWebhook().from_dict(item),
                )
            cleaned.append(typing.cast(MutatingWebhook, item))
        self._properties["webhooks"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the MutatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "create_namespaced_mutating_webhook_configuration",
            "create_mutating_webhook_configuration",
        ]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the MutatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "replace_namespaced_mutating_webhook_configuration",
            "replace_mutating_webhook_configuration",
        ]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the MutatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "patch_namespaced_mutating_webhook_configuration",
            "patch_mutating_webhook_configuration",
        ]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the MutatingWebhookConfiguration from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_mutating_webhook_configuration",
            "read_mutating_webhook_configuration",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the MutatingWebhookConfiguration from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_mutating_webhook_configuration",
            "delete_mutating_webhook_configuration",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.AdmissionregistrationV1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.AdmissionregistrationV1Api(**kwargs)

    def __enter__(self) -> "MutatingWebhookConfiguration":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class MutatingWebhookConfigurationList(_kuber_definitions.Collection):
    """
    MutatingWebhookConfigurationList is a list of
    MutatingWebhookConfiguration.
    """

    def __init__(
        self,
        items: typing.List["MutatingWebhookConfiguration"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create MutatingWebhookConfigurationList instance."""
        super(MutatingWebhookConfigurationList, self).__init__(
            api_version="admissionregistration/v1",
            kind="MutatingWebhookConfigurationList",
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, MutatingWebhookConfiguration),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["MutatingWebhookConfiguration"]:
        """
        List of MutatingWebhookConfiguration.
        """
        return typing.cast(
            typing.List["MutatingWebhookConfiguration"],
            self._properties.get("items"),
        )

    @items.setter
    def items(
        self,
        value: typing.Union[
            typing.List["MutatingWebhookConfiguration"], typing.List[dict]
        ],
    ):
        """
        List of MutatingWebhookConfiguration.
        """
        cleaned: typing.List[MutatingWebhookConfiguration] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    MutatingWebhookConfiguration,
                    MutatingWebhookConfiguration().from_dict(item),
                )
            cleaned.append(typing.cast(MutatingWebhookConfiguration, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#types-kinds
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#types-kinds
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.AdmissionregistrationV1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.AdmissionregistrationV1Api(**kwargs)

    def __enter__(self) -> "MutatingWebhookConfigurationList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class RuleWithOperations(_kuber_definitions.Definition):
    """
    RuleWithOperations is a tuple of Operations and Resources.
    It is recommended to make sure that all the tuple expansions
    are valid.
    """

    def __init__(
        self,
        api_groups: typing.List[str] = None,
        api_versions: typing.List[str] = None,
        operations: typing.List[str] = None,
        resources: typing.List[str] = None,
        scope: str = None,
    ):
        """Create RuleWithOperations instance."""
        super(RuleWithOperations, self).__init__(
            api_version="admissionregistration/v1", kind="RuleWithOperations"
        )
        self._properties = {
            "apiGroups": api_groups if api_groups is not None else [],
            "apiVersions": api_versions if api_versions is not None else [],
            "operations": operations if operations is not None else [],
            "resources": resources if resources is not None else [],
            "scope": scope if scope is not None else "",
        }
        self._types = {
            "apiGroups": (list, str),
            "apiVersions": (list, str),
            "operations": (list, str),
            "resources": (list, str),
            "scope": (str, None),
        }

    @property
    def api_groups(self) -> typing.List[str]:
        """
        APIGroups is the API groups the resources belong to. '*' is
        all groups. If '*' is present, the length of the slice must
        be one. Required.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("apiGroups"),
        )

    @api_groups.setter
    def api_groups(self, value: typing.List[str]):
        """
        APIGroups is the API groups the resources belong to. '*' is
        all groups. If '*' is present, the length of the slice must
        be one. Required.
        """
        self._properties["apiGroups"] = value

    @property
    def api_versions(self) -> typing.List[str]:
        """
        APIVersions is the API versions the resources belong to. '*'
        is all versions. If '*' is present, the length of the slice
        must be one. Required.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("apiVersions"),
        )

    @api_versions.setter
    def api_versions(self, value: typing.List[str]):
        """
        APIVersions is the API versions the resources belong to. '*'
        is all versions. If '*' is present, the length of the slice
        must be one. Required.
        """
        self._properties["apiVersions"] = value

    @property
    def operations(self) -> typing.List[str]:
        """
        Operations is the operations the admission hook cares about
        - CREATE, UPDATE, DELETE, CONNECT or * for all of those
        operations and any future admission operations that are
        added. If '*' is present, the length of the slice must be
        one. Required.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("operations"),
        )

    @operations.setter
    def operations(self, value: typing.List[str]):
        """
        Operations is the operations the admission hook cares about
        - CREATE, UPDATE, DELETE, CONNECT or * for all of those
        operations and any future admission operations that are
        added. If '*' is present, the length of the slice must be
        one. Required.
        """
        self._properties["operations"] = value

    @property
    def resources(self) -> typing.List[str]:
        """
        Resources is a list of resources this rule applies to.

        For example: 'pods' means pods. 'pods/log' means the log
        subresource of pods. '*' means all resources, but not
        subresources. 'pods/*' means all subresources of pods.
        '*/scale' means all scale subresources. '*/*' means all
        resources and their subresources.

        If wildcard is present, the validation rule will ensure
        resources do not overlap with each other.

        Depending on the enclosing object, subresources might not be
        allowed. Required.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("resources"),
        )

    @resources.setter
    def resources(self, value: typing.List[str]):
        """
        Resources is a list of resources this rule applies to.

        For example: 'pods' means pods. 'pods/log' means the log
        subresource of pods. '*' means all resources, but not
        subresources. 'pods/*' means all subresources of pods.
        '*/scale' means all scale subresources. '*/*' means all
        resources and their subresources.

        If wildcard is present, the validation rule will ensure
        resources do not overlap with each other.

        Depending on the enclosing object, subresources might not be
        allowed. Required.
        """
        self._properties["resources"] = value

    @property
    def scope(self) -> str:
        """
        scope specifies the scope of this rule. Valid values are
        "Cluster", "Namespaced", and "*" "Cluster" means that only
        cluster-scoped resources will match this rule. Namespace API
        objects are cluster-scoped. "Namespaced" means that only
        namespaced resources will match this rule. "*" means that
        there are no scope restrictions. Subresources match the
        scope of their parent resource. Default is "*".
        """
        return typing.cast(
            str,
            self._properties.get("scope"),
        )

    @scope.setter
    def scope(self, value: str):
        """
        scope specifies the scope of this rule. Valid values are
        "Cluster", "Namespaced", and "*" "Cluster" means that only
        cluster-scoped resources will match this rule. Namespace API
        objects are cluster-scoped. "Namespaced" means that only
        namespaced resources will match this rule. "*" means that
        there are no scope restrictions. Subresources match the
        scope of their parent resource. Default is "*".
        """
        self._properties["scope"] = value

    def __enter__(self) -> "RuleWithOperations":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ServiceReference(_kuber_definitions.Definition):
    """
    ServiceReference holds a reference to Service.legacy.k8s.io
    """

    def __init__(
        self,
        name: str = None,
        namespace: str = None,
        path: str = None,
        port: int = None,
    ):
        """Create ServiceReference instance."""
        super(ServiceReference, self).__init__(
            api_version="admissionregistration/v1", kind="ServiceReference"
        )
        self._properties = {
            "name": name if name is not None else "",
            "namespace": namespace if namespace is not None else "",
            "path": path if path is not None else "",
            "port": port if port is not None else None,
        }
        self._types = {
            "name": (str, None),
            "namespace": (str, None),
            "path": (str, None),
            "port": (int, None),
        }

    @property
    def name(self) -> str:
        """
        `name` is the name of the service. Required
        """
        return typing.cast(
            str,
            self._properties.get("name"),
        )

    @name.setter
    def name(self, value: str):
        """
        `name` is the name of the service. Required
        """
        self._properties["name"] = value

    @property
    def namespace(self) -> str:
        """
        `namespace` is the namespace of the service. Required
        """
        return typing.cast(
            str,
            self._properties.get("namespace"),
        )

    @namespace.setter
    def namespace(self, value: str):
        """
        `namespace` is the namespace of the service. Required
        """
        self._properties["namespace"] = value

    @property
    def path(self) -> str:
        """
        `path` is an optional URL path which will be sent in any
        request to this service.
        """
        return typing.cast(
            str,
            self._properties.get("path"),
        )

    @path.setter
    def path(self, value: str):
        """
        `path` is an optional URL path which will be sent in any
        request to this service.
        """
        self._properties["path"] = value

    @property
    def port(self) -> int:
        """
        If specified, the port on the service that hosting webhook.
        Default to 443 for backward compatibility. `port` should be
        a valid port number (1-65535, inclusive).
        """
        return typing.cast(
            int,
            self._properties.get("port"),
        )

    @port.setter
    def port(self, value: int):
        """
        If specified, the port on the service that hosting webhook.
        Default to 443 for backward compatibility. `port` should be
        a valid port number (1-65535, inclusive).
        """
        self._properties["port"] = value

    def __enter__(self) -> "ServiceReference":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ValidatingWebhook(_kuber_definitions.Definition):
    """
    ValidatingWebhook describes an admission webhook and the
    resources and operations it applies to.
    """

    def __init__(
        self,
        admission_review_versions: typing.List[str] = None,
        client_config: "WebhookClientConfig" = None,
        failure_policy: str = None,
        match_policy: str = None,
        name: str = None,
        namespace_selector: "LabelSelector" = None,
        object_selector: "LabelSelector" = None,
        rules: typing.List["RuleWithOperations"] = None,
        side_effects: str = None,
        timeout_seconds: int = None,
    ):
        """Create ValidatingWebhook instance."""
        super(ValidatingWebhook, self).__init__(
            api_version="admissionregistration/v1", kind="ValidatingWebhook"
        )
        self._properties = {
            "admissionReviewVersions": admission_review_versions
            if admission_review_versions is not None
            else [],
            "clientConfig": client_config
            if client_config is not None
            else WebhookClientConfig(),
            "failurePolicy": failure_policy if failure_policy is not None else "",
            "matchPolicy": match_policy if match_policy is not None else "",
            "name": name if name is not None else "",
            "namespaceSelector": namespace_selector
            if namespace_selector is not None
            else LabelSelector(),
            "objectSelector": object_selector
            if object_selector is not None
            else LabelSelector(),
            "rules": rules if rules is not None else [],
            "sideEffects": side_effects if side_effects is not None else "",
            "timeoutSeconds": timeout_seconds if timeout_seconds is not None else None,
        }
        self._types = {
            "admissionReviewVersions": (list, str),
            "clientConfig": (WebhookClientConfig, None),
            "failurePolicy": (str, None),
            "matchPolicy": (str, None),
            "name": (str, None),
            "namespaceSelector": (LabelSelector, None),
            "objectSelector": (LabelSelector, None),
            "rules": (list, RuleWithOperations),
            "sideEffects": (str, None),
            "timeoutSeconds": (int, None),
        }

    @property
    def admission_review_versions(self) -> typing.List[str]:
        """
        AdmissionReviewVersions is an ordered list of preferred
        `AdmissionReview` versions the Webhook expects. API server
        will try to use first version in the list which it supports.
        If none of the versions specified in this list supported by
        API server, validation will fail for this object. If a
        persisted webhook configuration specifies allowed versions
        and does not include any versions known to the API Server,
        calls to the webhook will fail and be subject to the failure
        policy.
        """
        return typing.cast(
            typing.List[str],
            self._properties.get("admissionReviewVersions"),
        )

    @admission_review_versions.setter
    def admission_review_versions(self, value: typing.List[str]):
        """
        AdmissionReviewVersions is an ordered list of preferred
        `AdmissionReview` versions the Webhook expects. API server
        will try to use first version in the list which it supports.
        If none of the versions specified in this list supported by
        API server, validation will fail for this object. If a
        persisted webhook configuration specifies allowed versions
        and does not include any versions known to the API Server,
        calls to the webhook will fail and be subject to the failure
        policy.
        """
        self._properties["admissionReviewVersions"] = value

    @property
    def client_config(self) -> "WebhookClientConfig":
        """
        ClientConfig defines how to communicate with the hook.
        Required
        """
        return typing.cast(
            "WebhookClientConfig",
            self._properties.get("clientConfig"),
        )

    @client_config.setter
    def client_config(self, value: typing.Union["WebhookClientConfig", dict]):
        """
        ClientConfig defines how to communicate with the hook.
        Required
        """
        if isinstance(value, dict):
            value = typing.cast(
                WebhookClientConfig,
                WebhookClientConfig().from_dict(value),
            )
        self._properties["clientConfig"] = value

    @property
    def failure_policy(self) -> str:
        """
        FailurePolicy defines how unrecognized errors from the
        admission endpoint are handled - allowed values are Ignore
        or Fail. Defaults to Fail.
        """
        return typing.cast(
            str,
            self._properties.get("failurePolicy"),
        )

    @failure_policy.setter
    def failure_policy(self, value: str):
        """
        FailurePolicy defines how unrecognized errors from the
        admission endpoint are handled - allowed values are Ignore
        or Fail. Defaults to Fail.
        """
        self._properties["failurePolicy"] = value

    @property
    def match_policy(self) -> str:
        """
        matchPolicy defines how the "rules" list is used to match
        incoming requests. Allowed values are "Exact" or
        "Equivalent".

        - Exact: match a request only if it exactly matches a
        specified rule. For example, if deployments can be modified
        via apps/v1, apps/v1beta1, and extensions/v1beta1, but
        "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request
        to apps/v1beta1 or extensions/v1beta1 would not be sent to
        the webhook.

        - Equivalent: match a request if modifies a resource listed
        in rules, even via another API group or version. For
        example, if deployments can be modified via apps/v1,
        apps/v1beta1, and extensions/v1beta1, and "rules" only
        included `apiGroups:["apps"], apiVersions:["v1"], resources:
        ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would be converted to apps/v1 and sent to
        the webhook.

        Defaults to "Equivalent"
        """
        return typing.cast(
            str,
            self._properties.get("matchPolicy"),
        )

    @match_policy.setter
    def match_policy(self, value: str):
        """
        matchPolicy defines how the "rules" list is used to match
        incoming requests. Allowed values are "Exact" or
        "Equivalent".

        - Exact: match a request only if it exactly matches a
        specified rule. For example, if deployments can be modified
        via apps/v1, apps/v1beta1, and extensions/v1beta1, but
        "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request
        to apps/v1beta1 or extensions/v1beta1 would not be sent to
        the webhook.

        - Equivalent: match a request if modifies a resource listed
        in rules, even via another API group or version. For
        example, if deployments can be modified via apps/v1,
        apps/v1beta1, and extensions/v1beta1, and "rules" only
        included `apiGroups:["apps"], apiVersions:["v1"], resources:
        ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would be converted to apps/v1 and sent to
        the webhook.

        Defaults to "Equivalent"
        """
        self._properties["matchPolicy"] = value

    @property
    def name(self) -> str:
        """
        The name of the admission webhook. Name should be fully
        qualified, e.g., imagepolicy.kubernetes.io, where
        "imagepolicy" is the name of the webhook, and kubernetes.io
        is the name of the organization. Required.
        """
        return typing.cast(
            str,
            self._properties.get("name"),
        )

    @name.setter
    def name(self, value: str):
        """
        The name of the admission webhook. Name should be fully
        qualified, e.g., imagepolicy.kubernetes.io, where
        "imagepolicy" is the name of the webhook, and kubernetes.io
        is the name of the organization. Required.
        """
        self._properties["name"] = value

    @property
    def namespace_selector(self) -> "LabelSelector":
        """
        NamespaceSelector decides whether to run the webhook on an
        object based on whether the namespace for that object
        matches the selector. If the object itself is a namespace,
        the matching is performed on object.metadata.labels. If the
        object is another cluster scoped resource, it never skips
        the webhook.

        For example, to run the webhook on any objects whose
        namespace is not associated with "runlevel" of "0" or "1";
        you will set the selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }

        If instead you want to only run the webhook on any objects
        whose namespace is associated with the "environment" of
        "prod" or "staging"; you will set the selector as follows:
        "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }

        See https://kubernetes.io/docs/concepts/overview/working-
        with-objects/labels for more examples of label selectors.

        Default to the empty LabelSelector, which matches
        everything.
        """
        return typing.cast(
            "LabelSelector",
            self._properties.get("namespaceSelector"),
        )

    @namespace_selector.setter
    def namespace_selector(self, value: typing.Union["LabelSelector", dict]):
        """
        NamespaceSelector decides whether to run the webhook on an
        object based on whether the namespace for that object
        matches the selector. If the object itself is a namespace,
        the matching is performed on object.metadata.labels. If the
        object is another cluster scoped resource, it never skips
        the webhook.

        For example, to run the webhook on any objects whose
        namespace is not associated with "runlevel" of "0" or "1";
        you will set the selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }

        If instead you want to only run the webhook on any objects
        whose namespace is associated with the "environment" of
        "prod" or "staging"; you will set the selector as follows:
        "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }

        See https://kubernetes.io/docs/concepts/overview/working-
        with-objects/labels for more examples of label selectors.

        Default to the empty LabelSelector, which matches
        everything.
        """
        if isinstance(value, dict):
            value = typing.cast(
                LabelSelector,
                LabelSelector().from_dict(value),
            )
        self._properties["namespaceSelector"] = value

    @property
    def object_selector(self) -> "LabelSelector":
        """
        ObjectSelector decides whether to run the webhook based on
        if the object has matching labels. objectSelector is
        evaluated against both the oldObject and newObject that
        would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject
        in the case of create, or newObject in the case of delete)
        or an object that cannot have labels (like a
        DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the
        webhook is opt-in, because end users may skip the admission
        webhook by setting the labels. Default to the empty
        LabelSelector, which matches everything.
        """
        return typing.cast(
            "LabelSelector",
            self._properties.get("objectSelector"),
        )

    @object_selector.setter
    def object_selector(self, value: typing.Union["LabelSelector", dict]):
        """
        ObjectSelector decides whether to run the webhook based on
        if the object has matching labels. objectSelector is
        evaluated against both the oldObject and newObject that
        would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject
        in the case of create, or newObject in the case of delete)
        or an object that cannot have labels (like a
        DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the
        webhook is opt-in, because end users may skip the admission
        webhook by setting the labels. Default to the empty
        LabelSelector, which matches everything.
        """
        if isinstance(value, dict):
            value = typing.cast(
                LabelSelector,
                LabelSelector().from_dict(value),
            )
        self._properties["objectSelector"] = value

    @property
    def rules(self) -> typing.List["RuleWithOperations"]:
        """
        Rules describes what operations on what
        resources/subresources the webhook cares about. The webhook
        cares about an operation if it matches _any_ Rule. However,
        in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a
        state which cannot be recovered from without completely
        disabling the plugin, ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks are never called on admission
        requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
        """
        return typing.cast(
            typing.List["RuleWithOperations"],
            self._properties.get("rules"),
        )

    @rules.setter
    def rules(
        self, value: typing.Union[typing.List["RuleWithOperations"], typing.List[dict]]
    ):
        """
        Rules describes what operations on what
        resources/subresources the webhook cares about. The webhook
        cares about an operation if it matches _any_ Rule. However,
        in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a
        state which cannot be recovered from without completely
        disabling the plugin, ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks are never called on admission
        requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
        """
        cleaned: typing.List[RuleWithOperations] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    RuleWithOperations,
                    RuleWithOperations().from_dict(item),
                )
            cleaned.append(typing.cast(RuleWithOperations, item))
        self._properties["rules"] = cleaned

    @property
    def side_effects(self) -> str:
        """
        SideEffects states whether this webhook has side effects.
        Acceptable values are: None, NoneOnDryRun (webhooks created
        via v1beta1 may also specify Some or Unknown). Webhooks with
        side effects MUST implement a reconciliation system, since a
        request may be rejected by a future step in the admission
        change and the side effects therefore need to be undone.
        Requests with the dryRun attribute will be auto-rejected if
        they match a webhook with sideEffects == Unknown or Some.
        """
        return typing.cast(
            str,
            self._properties.get("sideEffects"),
        )

    @side_effects.setter
    def side_effects(self, value: str):
        """
        SideEffects states whether this webhook has side effects.
        Acceptable values are: None, NoneOnDryRun (webhooks created
        via v1beta1 may also specify Some or Unknown). Webhooks with
        side effects MUST implement a reconciliation system, since a
        request may be rejected by a future step in the admission
        change and the side effects therefore need to be undone.
        Requests with the dryRun attribute will be auto-rejected if
        they match a webhook with sideEffects == Unknown or Some.
        """
        self._properties["sideEffects"] = value

    @property
    def timeout_seconds(self) -> int:
        """
        TimeoutSeconds specifies the timeout for this webhook. After
        the timeout passes, the webhook call will be ignored or the
        API call will fail based on the failure policy. The timeout
        value must be between 1 and 30 seconds. Default to 10
        seconds.
        """
        return typing.cast(
            int,
            self._properties.get("timeoutSeconds"),
        )

    @timeout_seconds.setter
    def timeout_seconds(self, value: int):
        """
        TimeoutSeconds specifies the timeout for this webhook. After
        the timeout passes, the webhook call will be ignored or the
        API call will fail based on the failure policy. The timeout
        value must be between 1 and 30 seconds. Default to 10
        seconds.
        """
        self._properties["timeoutSeconds"] = value

    def __enter__(self) -> "ValidatingWebhook":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ValidatingWebhookConfiguration(_kuber_definitions.Resource):
    """
    ValidatingWebhookConfiguration describes the configuration
    of and admission webhook that accept or reject and object
    without changing it.
    """

    def __init__(
        self,
        metadata: "ObjectMeta" = None,
        webhooks: typing.List["ValidatingWebhook"] = None,
    ):
        """Create ValidatingWebhookConfiguration instance."""
        super(ValidatingWebhookConfiguration, self).__init__(
            api_version="admissionregistration/v1",
            kind="ValidatingWebhookConfiguration",
        )
        self._properties = {
            "metadata": metadata if metadata is not None else ObjectMeta(),
            "webhooks": webhooks if webhooks is not None else [],
        }
        self._types = {
            "apiVersion": (str, None),
            "kind": (str, None),
            "metadata": (ObjectMeta, None),
            "webhooks": (list, ValidatingWebhook),
        }

    @property
    def metadata(self) -> "ObjectMeta":
        """
        Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata.
        """
        return typing.cast(
            "ObjectMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ObjectMeta", dict]):
        """
        Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#metadata.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ObjectMeta,
                ObjectMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @property
    def webhooks(self) -> typing.List["ValidatingWebhook"]:
        """
        Webhooks is a list of webhooks and the affected resources
        and operations.
        """
        return typing.cast(
            typing.List["ValidatingWebhook"],
            self._properties.get("webhooks"),
        )

    @webhooks.setter
    def webhooks(
        self, value: typing.Union[typing.List["ValidatingWebhook"], typing.List[dict]]
    ):
        """
        Webhooks is a list of webhooks and the affected resources
        and operations.
        """
        cleaned: typing.List[ValidatingWebhook] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    ValidatingWebhook,
                    ValidatingWebhook().from_dict(item),
                )
            cleaned.append(typing.cast(ValidatingWebhook, item))
        self._properties["webhooks"] = cleaned

    def create_resource(self, namespace: "str" = None):
        """
        Creates the ValidatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "create_namespaced_validating_webhook_configuration",
            "create_validating_webhook_configuration",
        ]

        _kube_api.execute(
            action="create",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict()},
        )

    def replace_resource(self, namespace: "str" = None):
        """
        Replaces the ValidatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "replace_namespaced_validating_webhook_configuration",
            "replace_validating_webhook_configuration",
        ]

        _kube_api.execute(
            action="replace",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def patch_resource(self, namespace: "str" = None):
        """
        Patches the ValidatingWebhookConfiguration in the currently
        configured Kubernetes cluster.
        """
        names = [
            "patch_namespaced_validating_webhook_configuration",
            "patch_validating_webhook_configuration",
        ]

        _kube_api.execute(
            action="patch",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"body": self.to_dict(), "name": self.metadata.name},
        )

    def get_resource_status(self, namespace: "str" = None):
        """This resource does not have a status."""
        pass

    def read_resource(self, namespace: str = None):
        """
        Reads the ValidatingWebhookConfiguration from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            "read_namespaced_validating_webhook_configuration",
            "read_validating_webhook_configuration",
        ]
        return _kube_api.execute(
            action="read",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name},
        )

    def delete_resource(
        self,
        namespace: str = None,
        propagation_policy: str = "Foreground",
        grace_period_seconds: int = 10,
    ):
        """
        Deletes the ValidatingWebhookConfiguration from the currently configured
        Kubernetes cluster.
        """
        names = [
            "delete_namespaced_validating_webhook_configuration",
            "delete_validating_webhook_configuration",
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds,
        )

        _kube_api.execute(
            action="delete",
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={"name": self.metadata.name, "body": body},
        )

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.AdmissionregistrationV1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.AdmissionregistrationV1Api(**kwargs)

    def __enter__(self) -> "ValidatingWebhookConfiguration":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class ValidatingWebhookConfigurationList(_kuber_definitions.Collection):
    """
    ValidatingWebhookConfigurationList is a list of
    ValidatingWebhookConfiguration.
    """

    def __init__(
        self,
        items: typing.List["ValidatingWebhookConfiguration"] = None,
        metadata: "ListMeta" = None,
    ):
        """Create ValidatingWebhookConfigurationList instance."""
        super(ValidatingWebhookConfigurationList, self).__init__(
            api_version="admissionregistration/v1",
            kind="ValidatingWebhookConfigurationList",
        )
        self._properties = {
            "items": items if items is not None else [],
            "metadata": metadata if metadata is not None else ListMeta(),
        }
        self._types = {
            "apiVersion": (str, None),
            "items": (list, ValidatingWebhookConfiguration),
            "kind": (str, None),
            "metadata": (ListMeta, None),
        }

    @property
    def items(self) -> typing.List["ValidatingWebhookConfiguration"]:
        """
        List of ValidatingWebhookConfiguration.
        """
        return typing.cast(
            typing.List["ValidatingWebhookConfiguration"],
            self._properties.get("items"),
        )

    @items.setter
    def items(
        self,
        value: typing.Union[
            typing.List["ValidatingWebhookConfiguration"], typing.List[dict]
        ],
    ):
        """
        List of ValidatingWebhookConfiguration.
        """
        cleaned: typing.List[ValidatingWebhookConfiguration] = []
        for item in value:
            if isinstance(item, dict):
                item = typing.cast(
                    ValidatingWebhookConfiguration,
                    ValidatingWebhookConfiguration().from_dict(item),
                )
            cleaned.append(typing.cast(ValidatingWebhookConfiguration, item))
        self._properties["items"] = cleaned

    @property
    def metadata(self) -> "ListMeta":
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#types-kinds
        """
        return typing.cast(
            "ListMeta",
            self._properties.get("metadata"),
        )

    @metadata.setter
    def metadata(self, value: typing.Union["ListMeta", dict]):
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-
        architecture/api-conventions.md#types-kinds
        """
        if isinstance(value, dict):
            value = typing.cast(
                ListMeta,
                ListMeta().from_dict(value),
            )
        self._properties["metadata"] = value

    @staticmethod
    def get_resource_api(
        api_client: client.ApiClient = None, **kwargs
    ) -> "client.AdmissionregistrationV1Api":
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs["apl_client"] = api_client
        return client.AdmissionregistrationV1Api(**kwargs)

    def __enter__(self) -> "ValidatingWebhookConfigurationList":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class WebhookClientConfig(_kuber_definitions.Definition):
    """
    WebhookClientConfig contains the information to make a TLS
    connection with the webhook
    """

    def __init__(
        self,
        ca_bundle: str = None,
        service: "ServiceReference" = None,
        url: str = None,
    ):
        """Create WebhookClientConfig instance."""
        super(WebhookClientConfig, self).__init__(
            api_version="admissionregistration/v1", kind="WebhookClientConfig"
        )
        self._properties = {
            "caBundle": ca_bundle if ca_bundle is not None else "",
            "service": service if service is not None else ServiceReference(),
            "url": url if url is not None else "",
        }
        self._types = {
            "caBundle": (str, None),
            "service": (ServiceReference, None),
            "url": (str, None),
        }

    @property
    def ca_bundle(self) -> str:
        """
        `caBundle` is a PEM encoded CA bundle which will be used to
        validate the webhook's server certificate. If unspecified,
        system trust roots on the apiserver are used.
        """
        return typing.cast(
            str,
            self._properties.get("caBundle"),
        )

    @ca_bundle.setter
    def ca_bundle(self, value: str):
        """
        `caBundle` is a PEM encoded CA bundle which will be used to
        validate the webhook's server certificate. If unspecified,
        system trust roots on the apiserver are used.
        """
        self._properties["caBundle"] = value

    @property
    def service(self) -> "ServiceReference":
        """
        `service` is a reference to the service for this webhook.
        Either `service` or `url` must be specified.

        If the webhook is running within the cluster, then you
        should use `service`.
        """
        return typing.cast(
            "ServiceReference",
            self._properties.get("service"),
        )

    @service.setter
    def service(self, value: typing.Union["ServiceReference", dict]):
        """
        `service` is a reference to the service for this webhook.
        Either `service` or `url` must be specified.

        If the webhook is running within the cluster, then you
        should use `service`.
        """
        if isinstance(value, dict):
            value = typing.cast(
                ServiceReference,
                ServiceReference().from_dict(value),
            )
        self._properties["service"] = value

    @property
    def url(self) -> str:
        """
        `url` gives the location of the webhook, in standard URL
        form (`scheme://host:port/path`). Exactly one of `url` or
        `service` must be specified.

        The `host` should not refer to a service running in the
        cluster; use the `service` field instead. The host might be
        resolved via external DNS in some apiservers (e.g., `kube-
        apiserver` cannot resolve in-cluster DNS as that would be a
        layering violation). `host` may also be an IP address.

        Please note that using `localhost` or `127.0.0.1` as a
        `host` is risky unless you take great care to run this
        webhook on all hosts which run an apiserver which might need
        to make calls to this webhook. Such installs are likely to
        be non-portable, i.e., not easy to turn up in a new cluster.

        The scheme must be "https"; the URL must begin with
        "https://".

        A path is optional, and if present may be any string
        permissible in a URL. You may use the path to pass an
        arbitrary string to the webhook, for example, a cluster
        identifier.

        Attempting to use a user or basic auth e.g. "user:password@"
        is not allowed. Fragments ("#...") and query parameters
        ("?...") are not allowed, either.
        """
        return typing.cast(
            str,
            self._properties.get("url"),
        )

    @url.setter
    def url(self, value: str):
        """
        `url` gives the location of the webhook, in standard URL
        form (`scheme://host:port/path`). Exactly one of `url` or
        `service` must be specified.

        The `host` should not refer to a service running in the
        cluster; use the `service` field instead. The host might be
        resolved via external DNS in some apiservers (e.g., `kube-
        apiserver` cannot resolve in-cluster DNS as that would be a
        layering violation). `host` may also be an IP address.

        Please note that using `localhost` or `127.0.0.1` as a
        `host` is risky unless you take great care to run this
        webhook on all hosts which run an apiserver which might need
        to make calls to this webhook. Such installs are likely to
        be non-portable, i.e., not easy to turn up in a new cluster.

        The scheme must be "https"; the URL must begin with
        "https://".

        A path is optional, and if present may be any string
        permissible in a URL. You may use the path to pass an
        arbitrary string to the webhook, for example, a cluster
        identifier.

        Attempting to use a user or basic auth e.g. "user:password@"
        is not allowed. Fragments ("#...") and query parameters
        ("?...") are not allowed, either.
        """
        self._properties["url"] = value

    def __enter__(self) -> "WebhookClientConfig":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False
